#set -x
check_CERT() {
  if [ -e /etc/hiveon-pub.pem ]; then
    openssl dgst -sha256 -verify /etc/hiveon-pub.pem -signature  cert.pem.sig  cert.pem >/dev/null  2>&1 
    res=$?
    if [ $res -ne 0 ]; then
      echo "RUNME: CANNOT VERIFY CERT !" >> /tmp/upgrade_result
      rm * -rf > /dev/null 2>&1
      exit 3
    fi
  fi
  if [ ! -f fw.tar.gz.sig ]; then
    echo "RUNME: CANNOT FIND FW SIGNATURE!" >> /tmp/upgrade_result
    rm * -rf > /dev/null 2>&1
    exit 4
  fi
  openssl dgst -sha256 -verify cert.pem -signature  fw.tar.gz.sig  fw.tar.gz >/dev/null  2>&1 
  res=$?
  if [ $res -ne 0 ]; then
    echo "RUNME: CANNOT VERIFY FW !" >> /tmp/upgrade_result
    rm * -rf > /dev/null 2>&1
    exit 5
  fi
}

check_CB() {
  CB=""
  if [ -e /usr/bin/manufacturer ]; then
    CB="$(cat /usr/bin/manufacturer)"
  fi
  if [ "$CB" == "AM335x BeagleBone" ]; then
    echo "RUNME: CONTROL BOARD IS NOT XILINX" >> /tmp/upgrade_result
    exit 5
  fi
}

check_SD() {
  if [ -e /dev/mmcblk0p1 ]; then
    SD="/dev/mmcblk0p1"
  elif [ -e /dev/mmcblk0 ]; then
    SD="/dev/mmcblk0"
  else
    echo "Error SD card!" >> /tmp/upgrade_result
  fi
}

check_MOUNT() {
  umount -f /dev/mmcblk* > /dev/null 2>&1 # for multiple mount
  mount | grep mmc > /dev/null;
  res_mount=$?
  if [ $res_mount -eq 0 ]; then
    echo "RUNME: CANNOT UNMOUNT SD, PLEASE REBOOT !" >> /tmp/upgrade_result
    exit 5
  fi

  while mount | grep volatile &>/dev/null; do # umount tmpfs on /tmp
    mount_tmpfs="$(mount | grep volatile | awk 'NR==1{print $3}')"
    umount -l "$mount_tmpfs" >/dev/null 2>&1
    sleep 1
  done
}

check_MEM() {
#  init q
#  killall perl >/dev/null 2>&1
#  killall monitorcg >/dev/null 2>&1
#  /etc/init.d/cgminer.sh stop >/dev/null 2>&1
#
  sh /www/pages/cgi-bin/stop_miner.cgi >/dev/null 2>&1
  sync; echo 1 > /proc/sys/vm/drop_caches >/dev/null 2>&1
  sleep 10
  if [ "$(grep MemFree /proc/meminfo | awk '{print $2}')" -lt 60000 ]; then
    echo "RUNME: LOW MEMORY, PLEASE REBOOT !" >> /tmp/upgrade_result
    exit 5
  fi
}

start_MOUNT() {
  [ ! -d "/mnt/mmc" ] && mkdir -p /mnt/mmc
  mount -t vfat -o sync "$SD" /mnt/mmc
}

stop_MOUNT() {
  busy=true
  umount "$SD" >/dev/null 2>&1
  while $busy
  do
  if mountpoint -q "$SD"; then
    umount "$SD" >/dev/null 2>&1
    if [ $? -eq 0 ]; then
     busy=false
    else
     sleep 5
    fi
  else
    busy=false
  fi
  done
  sleep 10
  rm -f /config/norespawn >/dev/null 2>&1
  sync >/dev/null 2>&1
}

check_FILES() {
  if ! cmp -s /mnt/mmc/devicetree.dtb devicetree.dtb ; then
    echo "RUNME: FILES ARE PREPARED, REBOOTING..." >> /tmp/upgrade_result
    cp -f devicetree.dtb /mnt/mmc/devicetree.dtb
    /hive/bin/message info "Rebooting... Please repeat the update after the start" > /dev/null 2>&1
    sleep 10
    stop_MOUNT
    reboot
  fi
}

start_RUNME() {
  [ -d "/mnt/mmc/fw" ] && rm -rf /mnt/mmc/fw
  mkdir -p /mnt/mmc/fw
  tar zxvf fw.tar.gz -C /mnt/mmc/fw > /dev/null 2>&1
  rm fw.tar.gz
  cd /mnt/mmc/fw
  sh runme.sh
  cd /tmp
  sync >/dev/null 2>&1
}

check_CERT
check_CB
check_SD
check_MOUNT
check_MEM
start_MOUNT
check_FILES
start_RUNME
stop_MOUNT


#end